@@ -1,5 +1,5 @@
 /*
- * $Id: dbus-client.h 259 2007-02-22 00:13:15Z jblache $
+ * $Id: dbus-client.h 341 2007-07-26 15:49:44Z jblache $
  */
 #ifndef __MBP_DBUS_CLIENT_H__
 #define __MBP_DBUS_CLIENT_H__
@@ -14,6 +14,9 @@
 #define MBP_DBUS_SIG_LIGHT  (1 << 5)
 
 
+#define LCD_USER      0
+#define LCD_AUTO      1
+
 #define KBD_USER      0
 #define KBD_AUTO      1
 
