@@ -1,5 +1,5 @@
 /*
- * $Id: conffile.h 277 2007-02-23 19:57:25Z jblache $
+ * $Id: conffile.h 345 2007-07-27 14:00:46Z jblache $
  */
 
 #ifndef __CONFFILE_H__
@@ -13,6 +13,7 @@ struct _general_cfg {
 struct _lcd_sysfs_cfg {
   int init;
   int step;
+  int on_batt;
 };
 
 #else
@@ -20,12 +21,21 @@ struct _lcd_sysfs_cfg {
 struct _lcd_x1600_cfg {
   int init;
   int step;
+  int on_batt;
 };
 
 struct _lcd_gma950_cfg {
   unsigned int init;
   unsigned int step;
+  unsigned int on_batt;
 };
+
+struct _lcd_nv8600mgt_cfg {
+  int init;
+  int step;
+  int on_batt;
+};
+
 #endif /* __powerpc__ */
 
 struct _audio_cfg {
@@ -60,10 +70,10 @@ struct _appleir_cfg {
 extern struct _general_cfg general_cfg;
 #ifdef __powerpc__
 extern struct _lcd_sysfs_cfg lcd_sysfs_cfg;
-extern struct _lcd_r128_cfg lcd_r128_cfg;
 #else
 extern struct _lcd_x1600_cfg lcd_x1600_cfg;
 extern struct _lcd_gma950_cfg lcd_gma950_cfg;
+extern struct _lcd_nv8600mgt_cfg lcd_nv8600mgt_cfg;
 #endif
 extern struct _audio_cfg audio_cfg;
 extern struct _kbd_cfg kbd_cfg;
