@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: dbus.c 259 2007-02-22 00:13:15Z jblache $
+ * $Id: dbus.c 341 2007-07-26 15:49:44Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -31,6 +31,7 @@
 #include "kbd_backlight.h"
 #include "ambient.h"
 #include "audio.h"
+#include "cd_eject.h"
 
 
 static DBusError err;
@@ -38,7 +39,7 @@ static DBusConnection *conn;
 
 
 void
-mbpdbus_send_lcd_backlight(int cur, int prev)
+mbpdbus_send_lcd_backlight(int cur, int prev, int who)
 {
   DBusMessage *msg;
 
@@ -63,6 +64,7 @@ mbpdbus_send_lcd_backlight(int cur, int prev)
 				 DBUS_TYPE_UINT32, &cur,
 				 DBUS_TYPE_UINT32, &prev,
 				 DBUS_TYPE_UINT32, &lcd_bck_info.max,
+				 DBUS_TYPE_UINT32, &who,
 				 DBUS_TYPE_INVALID);
   if (ret == FALSE)
     {
@@ -332,7 +334,7 @@ mbpdbus_send_cd_eject(void)
 
 
 static void
-process_lcd_level_call(DBusMessage *req)
+process_lcd_getlevel_call(DBusMessage *req)
 {
   DBusMessage *msg;
   DBusMessageIter args;
@@ -378,7 +380,7 @@ process_lcd_level_call(DBusMessage *req)
 
 
 static void
-process_kbd_level_call(DBusMessage *req)
+process_kbd_getlevel_call(DBusMessage *req)
 {
   DBusMessage *msg;
   DBusMessageIter args;
@@ -424,7 +426,7 @@ process_kbd_level_call(DBusMessage *req)
 
 
 static void
-process_ambient_level_call(DBusMessage *req)
+process_ambient_getlevel_call(DBusMessage *req)
 {
   DBusMessage *msg;
   DBusMessageIter args;
@@ -470,7 +472,7 @@ process_ambient_level_call(DBusMessage *req)
 }
 
 static void
-process_audio_volume_call(DBusMessage *req)
+process_audio_getvolume_call(DBusMessage *req)
 {
   DBusMessage *msg;
   DBusMessageIter args;
@@ -514,9 +516,8 @@ process_audio_volume_call(DBusMessage *req)
   dbus_message_unref(msg);
 }
 
-
 static void
-process_audio_mute_call(DBusMessage *req)
+process_audio_getmute_call(DBusMessage *req)
 {
   DBusMessage *msg;
   DBusMessageIter args;
@@ -560,6 +561,140 @@ process_audio_mute_call(DBusMessage *req)
 }
 
 
+static void
+process_lcd_backlight_step_call(DBusMessage *req, int dir)
+{
+  DBusMessage *msg;
+
+  int ret;
+
+  logdebug("Got lcdBacklight levelUp/levelDown call\n");
+
+  mops->lcd_backlight_step(dir);
+
+  msg = dbus_message_new_method_return(req);
+
+  ret = dbus_connection_send(conn, msg, NULL);
+  if (ret == FALSE)
+    {
+      logdebug("Could not send lcdBacklight levelUp/levelDown reply\n");
+
+      dbus_message_unref(msg);
+
+      return;
+    }
+
+  dbus_message_unref(msg);
+}
+
+static void
+process_kbd_backlight_inhibit_call(DBusMessage *req, int inhibit)
+{
+  DBusMessage *msg;
+
+  int ret;
+
+  logdebug("Got kbdBacklight inhibit call\n");
+
+  if (inhibit)
+    kbd_backlight_inhibit_set(KBD_INHIBIT_USER);
+  else
+    kbd_backlight_inhibit_clear(KBD_INHIBIT_USER);
+
+  msg = dbus_message_new_method_return(req);
+
+  ret = dbus_connection_send(conn, msg, NULL);
+  if (ret == FALSE)
+    {
+      logdebug("Could not send kbdBacklight inhibit reply\n");
+
+      dbus_message_unref(msg);
+
+      return;
+    }
+
+  dbus_message_unref(msg);
+}
+
+static void
+process_audio_volume_step_call(DBusMessage *req, int dir)
+{
+  DBusMessage *msg;
+
+  int ret;
+
+  logdebug("Got audio volumeUp/volumeDown call\n");
+
+  audio_step(dir);
+
+  msg = dbus_message_new_method_return(req);
+
+  ret = dbus_connection_send(conn, msg, NULL);
+  if (ret == FALSE)
+    {
+      logdebug("Could not send audio volumeUp/volumeDown reply\n");
+
+      dbus_message_unref(msg);
+
+      return;
+    }
+
+  dbus_message_unref(msg);
+}
+
+static void
+process_audio_toggle_mute_call(DBusMessage *req)
+{
+  DBusMessage *msg;
+
+  int ret;
+
+  logdebug("Got audio toggleMute call\n");
+
+  audio_toggle_mute();
+
+  msg = dbus_message_new_method_return(req);
+
+  ret = dbus_connection_send(conn, msg, NULL);
+  if (ret == FALSE)
+    {
+      logdebug("Could not send audio toggleMute reply\n");
+
+      dbus_message_unref(msg);
+
+      return;
+    }
+
+  dbus_message_unref(msg);
+}
+
+static void
+process_cd_eject_call(DBusMessage *req)
+{
+  DBusMessage *msg;
+
+  int ret;
+
+  logdebug("Got cd eject call\n");
+
+  cd_eject();
+
+  msg = dbus_message_new_method_return(req);
+
+  ret = dbus_connection_send(conn, msg, NULL);
+  if (ret == FALSE)
+    {
+      logdebug("Could not send cd eject reply\n");
+
+      dbus_message_unref(msg);
+
+      return;
+    }
+
+  dbus_message_unref(msg);
+}
+
+
 void
 mbpdbus_process_requests(void)
 {
@@ -589,16 +724,34 @@ mbpdbus_process_requests(void)
 	  break;
 	}
 
+      // Get methods
       if (dbus_message_is_method_call(msg, "org.pommed.lcdBacklight", "getLevel"))
-	process_lcd_level_call(msg);
+	process_lcd_getlevel_call(msg);
       else if (dbus_message_is_method_call(msg, "org.pommed.kbdBacklight", "getLevel"))
-	process_kbd_level_call(msg);
+	process_kbd_getlevel_call(msg);
       else if (dbus_message_is_method_call(msg, "org.pommed.ambient", "getLevel"))
-	process_ambient_level_call(msg);
+	process_ambient_getlevel_call(msg);
       else if (dbus_message_is_method_call(msg, "org.pommed.audio", "getVolume"))
-	process_audio_volume_call(msg);
+	process_audio_getvolume_call(msg);
       else if (dbus_message_is_method_call(msg, "org.pommed.audio", "getMute"))
-	process_audio_mute_call(msg);
+	process_audio_getmute_call(msg);
+      // Set methods
+      else if (dbus_message_is_method_call(msg, "org.pommed.lcdBacklight", "levelUp"))
+        process_lcd_backlight_step_call(msg, STEP_UP);
+      else if (dbus_message_is_method_call(msg, "org.pommed.lcdBacklight", "levelDown"))
+        process_lcd_backlight_step_call(msg, STEP_DOWN);
+      else if (dbus_message_is_method_call(msg, "org.pommed.kbdBacklight", "inhibit"))
+        process_kbd_backlight_inhibit_call(msg, 1);
+      else if (dbus_message_is_method_call(msg, "org.pommed.kbdBacklight", "disinhibit"))
+        process_kbd_backlight_inhibit_call(msg, 0);
+      else if (dbus_message_is_method_call(msg, "org.pommed.audio", "volumeUp"))
+        process_audio_volume_step_call(msg, STEP_UP);
+      else if (dbus_message_is_method_call(msg, "org.pommed.audio", "volumeDown"))
+        process_audio_volume_step_call(msg, STEP_DOWN);
+      else if (dbus_message_is_method_call(msg, "org.pommed.audio", "toggleMute"))
+        process_audio_toggle_mute_call(msg);
+      else if (dbus_message_is_method_call(msg, "org.pommed.cd", "eject"))
+        process_cd_eject_call(msg);
       else if (dbus_message_is_signal(msg, DBUS_INTERFACE_LOCAL, "Disconnected"))
 	{
 	  logmsg(LOG_INFO, "DBus disconnected");
