@@ -1,5 +1,5 @@
 /*
- * $Id: dbus.h 259 2007-02-22 00:13:15Z jblache $
+ * $Id: dbus.h 341 2007-07-26 15:49:44Z jblache $
  */
 
 #ifndef __MBPDBUS_H__
@@ -7,7 +7,7 @@
 
 
 void
-mbpdbus_send_lcd_backlight(int cur, int prev);
+mbpdbus_send_lcd_backlight(int cur, int prev, int who);
 
 void
 mbpdbus_send_kbd_backlight(int cur, int prev, int who);
