@@ -1,9 +1,9 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: evdev.c 289 2007-03-02 19:32:38Z jblache $
+ * $Id: evdev.c 336 2007-06-17 08:44:45Z jblache $
  *
- * Copyright (C) 2006 Julien BLACHE <jb@jblache.org>
+ * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,6 +56,19 @@
 #endif
 
 
+#define EVDEV_ADB_KBD       (1 << 0)
+#define EVDEV_ADB_BUTTONS   (1 << 1)
+#define EVDEV_USB_KBD1      (1 << 2)
+#define EVDEV_USB_KBD2      (1 << 3)
+#define EVDEV_APPLEIR       (1 << 5)
+#define EVDEV_MOUSEEMU      (1 << 6)
+#define EVDEV_SW_LID        (1 << 7)
+
+#define EVDEV_NO_EVDEV      0
+
+static unsigned int evdevs;
+
+
 void
 evdev_process_events(int fd)
 {
@@ -116,7 +129,10 @@ evdev_process_events(int fd)
 	    if (!has_kbd_backlight())
 	      break;
 
-	    kbd_backlight_inhibit_toggle(KBD_INHIBIT_USER);
+	    if (kbd_cfg.auto_on)
+	      kbd_backlight_inhibit_toggle(KBD_INHIBIT_USER);
+	    else
+	      kbd_backlight_toggle();
 	    break;
 
 	  case K_KBD_BCK_DOWN:
@@ -200,8 +216,23 @@ evdev_is_adb(unsigned short *id)
   if (id[ID_VENDOR] != 0x0001)
     return 0;
 
-  return ((product == ADB_PRODUCT_ID_KEYBOARD)
-	  || (product == ADB_PRODUCT_ID_PBBUTTONS));
+  if (product == ADB_PRODUCT_ID_KEYBOARD)
+    {
+      logdebug(" -> ADB keyboard\n");
+
+      evdevs |= EVDEV_ADB_KBD;
+      return 1;
+    }
+
+  if (product == ADB_PRODUCT_ID_PBBUTTONS)
+    {
+      logdebug(" -> ADB PowerBook buttons\n");
+
+      evdevs |= EVDEV_ADB_BUTTONS;
+      return 1;
+    }
+
+  return 0;
 }
 
 /* PowerBook G4 */
@@ -216,9 +247,49 @@ evdev_is_fountain(unsigned short *id)
   if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
     return 0;
 
-  return ((product == USB_PRODUCT_ID_FOUNTAIN_ANSI)
-	  || (product == USB_PRODUCT_ID_FOUNTAIN_ISO)
-	  || (product == USB_PRODUCT_ID_FOUNTAIN_JIS));
+  if ((product == USB_PRODUCT_ID_FOUNTAIN_ANSI)
+      || (product == USB_PRODUCT_ID_FOUNTAIN_ISO)
+      || (product == USB_PRODUCT_ID_FOUNTAIN_JIS))
+    {
+      logdebug(" -> Fountain USB keyboard\n");
+
+      if (evdevs & EVDEV_USB_KBD1)
+	evdevs |= EVDEV_USB_KBD2;
+      else
+	evdevs |= EVDEV_USB_KBD1;
+
+      return 1;
+    }
+
+  return 0;
+}
+
+int
+evdev_is_geyser(unsigned short *id)
+{
+  unsigned short product = id[ID_PRODUCT];
+
+  if (id[ID_BUS] != BUS_USB)
+    return 0;
+
+  if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
+    return 0;
+
+  if ((product == USB_PRODUCT_ID_GEYSER_ANSI)
+      || (product == USB_PRODUCT_ID_GEYSER_ISO)
+      || (product == USB_PRODUCT_ID_GEYSER_JIS))
+    {
+      logdebug(" -> Geyser USB keyboard\n");
+
+      if (evdevs & EVDEV_USB_KBD1)
+	evdevs |= EVDEV_USB_KBD2;
+      else
+	evdevs |= EVDEV_USB_KBD1;
+
+      return 1;
+    }
+
+  return 0;
 }
 
 /* PMU Lid switch */
@@ -236,7 +307,15 @@ evdev_is_lidswitch(unsigned short *id)
   if (id[ID_VERSION] != 0x0100)
     return 0;
 
-  return (product == 0x0001);
+  if (product == 0x0001)
+    {
+      logdebug(" -> PMU LID switch\n");
+
+      evdevs |= EVDEV_SW_LID;
+      return 1;
+    }
+
+  return 0;
 }
 
 #else
@@ -253,9 +332,21 @@ evdev_is_geyser3(unsigned short *id)
   if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
     return 0;
 
-  return ((product == USB_PRODUCT_ID_GEYSER3_ANSI)
-	  || (product == USB_PRODUCT_ID_GEYSER3_ISO)
-	  || (product == USB_PRODUCT_ID_GEYSER3_JIS));
+  if ((product == USB_PRODUCT_ID_GEYSER3_ANSI)
+      || (product == USB_PRODUCT_ID_GEYSER3_ISO)
+      || (product == USB_PRODUCT_ID_GEYSER3_JIS))
+    {
+      logdebug(" -> Geyser III USB keyboard\n");
+
+      if (evdevs & EVDEV_USB_KBD1)
+	evdevs |= EVDEV_USB_KBD2;
+      else
+	evdevs |= EVDEV_USB_KBD1;
+
+      return 1;
+    }
+
+  return 0;
 }
 
 /* Core2 Duo MacBook & MacBook Pro */
@@ -270,9 +361,21 @@ evdev_is_geyser4(unsigned short *id)
   if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
     return 0;
 
-  return ((product == USB_PRODUCT_ID_GEYSER4_ANSI)
-	  || (product == USB_PRODUCT_ID_GEYSER4_ISO)
-	  || (product == USB_PRODUCT_ID_GEYSER4_JIS));
+  if ((product == USB_PRODUCT_ID_GEYSER4_ANSI)
+      || (product == USB_PRODUCT_ID_GEYSER4_ISO)
+      || (product == USB_PRODUCT_ID_GEYSER4_JIS))
+    {
+      logdebug(" -> Geyser IV USB keyboard\n");
+
+      if (evdevs & EVDEV_USB_KBD1)
+	evdevs |= EVDEV_USB_KBD2;
+      else
+	evdevs |= EVDEV_USB_KBD1;
+
+      return 1;
+    }
+
+  return 0;
 }
 
 /* Apple Remote IR Receiver */
@@ -287,7 +390,16 @@ evdev_is_appleir(unsigned short *id)
   if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
     return 0;
 
-  return (product == USB_PRODUCT_ID_APPLEIR);
+  if ((product == USB_PRODUCT_ID_APPLEIR)
+      || (product == USB_PRODUCT_ID_APPLEIR_2))
+    {
+      logdebug(" -> Apple IR receiver\n");
+
+      evdevs |= EVDEV_APPLEIR;
+      return 1;
+    }
+
+  return 0;
 }
 
 /* ACPI Lid switch */
@@ -302,7 +414,15 @@ evdev_is_lidswitch(unsigned short *id)
   if (id[ID_VENDOR] != 0)
     return 0;
 
-  return (product == 0x0005);
+  if (product == 0x0005)
+    {
+      logdebug(" -> ACPI LID switch\n");
+
+      evdevs |= EVDEV_SW_LID;
+      return 1;
+    }
+
+  return 0;
 }
 #endif /* !__powerpc__ */
 
@@ -318,7 +438,15 @@ evdev_is_mouseemu(unsigned short *id)
   if (id[ID_VENDOR] != 0x001f)
     return 0;
 
-  return (product == 0x001f);
+  if (product == 0x001f)
+    {
+      logdebug(" -> Mouseemu virtual keyboard\n");
+
+      evdevs |= EVDEV_MOUSEEMU;
+      return 1;
+    }
+
+  return 0;
 }
 
 
@@ -336,6 +464,8 @@ evdev_open(struct pollfd **fds)
   char devname[256];
   char evdev[32];
 
+  evdevs = EVDEV_NO_EVDEV;
+
   for (i = 0; i < EVDEV_MAX; i++)
     {
       ret = snprintf(evdev, 32, "%s%d", EVDEV_BASE, i);
@@ -354,7 +484,7 @@ evdev_open(struct pollfd **fds)
 
       devname[0] = '\0';
       ioctl(fd[i], EVIOCGNAME(sizeof(devname)), devname);
-      logdebug("Investigating evdev %d [%s]\n", i, devname);
+      logdebug("\nInvestigating evdev %d [%s]\n", i, devname);
 
 
       ioctl(fd[i], EVIOCGID, id);
@@ -366,7 +496,7 @@ evdev_open(struct pollfd **fds)
 	  && !(has_kbd_backlight() && evdev_is_lidswitch(id))
 	  && !(evdev_is_mouseemu(id)))
 	{
-	  logdebug("Discarding evdev %d vid 0x%04x, pid 0x%04x\n", i, id[ID_VENDOR], id[ID_PRODUCT]);
+	  logdebug("Discarding evdev %d: bus 0x%04x, vid 0x%04x, pid 0x%04x\n", i, id[ID_BUS], id[ID_VENDOR], id[ID_PRODUCT]);
 
 	  close(fd[i]);
 	  fd[i] = -1;
@@ -407,7 +537,7 @@ evdev_open(struct pollfd **fds)
       found++;
     }
 
-  logdebug("Found %d devices\n", found);
+  logdebug("\nFound %d devices\n", found);
 
   *fds = (struct pollfd *) malloc(found * sizeof(struct pollfd));
 
@@ -459,22 +589,32 @@ int
 evdev_reopen(struct pollfd **fds, int nfds)
 {
   int i;
+  unsigned int prev_evdevs = evdevs;
 
   evdev_close(fds, nfds);
 
+  logdebug("Previous event devices: 0x%04x\n", prev_evdevs);
+
   /* When resuming, we need to reopen event devices which
    * disappear at suspend time. We need to wait for udev to
    * recreate the device nodes.
-   * Wait for up to 12 seconds, 24 * 0.5 seconds
    */
-  for (i = 0; i < 24; i++)
+  for (i = 0; i < 50; i++)
     {
       usleep(500000);
 
       nfds = evdev_open(fds);
 
       if (nfds > 0)
-	break;
+	{
+	  logdebug("Got event devices 0x%04x at iteration %d\n", prev_evdevs, i);
+
+	  if (evdevs == prev_evdevs)
+	    break;
+
+	  /* We haven't got all the event devices we need */
+	  evdev_close(fds, nfds);
+	}
     }
 
   return nfds;
