@@ -1,5 +1,5 @@
 /*
- * $Id: evdev.h 244 2007-01-09 18:01:00Z julien $
+ * $Id: evdev.h 336 2007-06-17 08:44:45Z jblache $
  */
 
 #ifndef __EVDEV_H__
@@ -23,6 +23,11 @@
 #define USB_PRODUCT_ID_FOUNTAIN_ISO   0x020f
 #define USB_PRODUCT_ID_FOUNTAIN_JIS   0x0210
 
+/* Apple Geyser keyboard + trackpad */
+#define USB_PRODUCT_ID_GEYSER_ANSI   0x0214
+#define USB_PRODUCT_ID_GEYSER_ISO    0x0215
+#define USB_PRODUCT_ID_GEYSER_JIS    0x0216
+
 /* Apple Geyser III keyboard + trackpad */
 #define USB_PRODUCT_ID_GEYSER3_ANSI   0x0217
 #define USB_PRODUCT_ID_GEYSER3_ISO    0x0218
@@ -35,6 +40,7 @@
 
 /* Apple Remote IR Receiver */
 #define USB_PRODUCT_ID_APPLEIR        0x8240
+#define USB_PRODUCT_ID_APPLEIR_2      0x8242
 
 
 /* Keyboard hotkeys */
@@ -72,6 +78,9 @@ evdev_is_adb(unsigned short *id);
 int
 evdev_is_fountain(unsigned short *id);
 
+int
+evdev_is_geyser(unsigned short *id);
+
 #else
 
 int
