@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: kbd_auto.c 291 2007-03-02 19:45:47Z jblache $
+ * $Id: kbd_auto.c 321 2007-05-05 16:44:26Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -19,6 +19,30 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+/* simple backlight toggle */
+void
+kbd_backlight_toggle(void)
+{
+  int curval;
+
+  curval = kbd_backlight_get();
+
+  if (curval != KBD_BACKLIGHT_OFF)
+    {
+      kbd_bck_info.toggle_lvl = curval;
+      kbd_backlight_set(KBD_BACKLIGHT_OFF, KBD_USER);
+    }
+  else
+    {
+      if (kbd_bck_info.toggle_lvl < kbd_cfg.auto_lvl)
+	kbd_bck_info.toggle_lvl = kbd_cfg.auto_lvl;
+
+      kbd_backlight_set(kbd_bck_info.toggle_lvl, KBD_USER);
+    }
+}
+
+
+/* Automatic backlight */
 void
 kbd_backlight_inhibit_set(int mask)
 {
