@@ -1,31 +1,25 @@
 /*
- * $Id: kbd_backlight.h 291 2007-03-02 19:45:47Z jblache $
+ * $Id: kbd_backlight.h 319 2007-05-03 08:28:44Z jblache $
  */
 
 #ifndef __KBD_BACKLIGHT_H__
 #define __KBD_BACKLIGHT_H__
 
-#ifdef __powerpc__
-int
-kbd_get_lmuaddr(void);
-
-char*
-kbd_get_i2cdev(int addr);
-
-int 
-kbd_probe_lmu(int addr, char* i2cdev);
-
-#else
+#ifndef __powerpc__
 #define KBD_BACKLIGHT           "/sys/class/leds/smc:kbd_backlight/brightness"
-#endif /* __powerpc__ */
+#endif /* !__powerpc__ */
 
 
 #define KBD_BACKLIGHT_OFF       0
 #define KBD_BACKLIGHT_MAX       255
 
+/* fading duration in milliseconds */
+#define KBD_BACKLIGHT_FADE_LENGTH 350
+#define KBD_BACKLIGHT_FADE_STEPS 20
 
 #define KBD_INHIBIT_USER        (1 << 0)
 #define KBD_INHIBIT_LID         (1 << 1)
+#define KBD_INHIBIT_CFG         (1 << 2)
 
 
 #define KBD_USER     0
@@ -40,6 +34,8 @@ struct _kbd_bck_info
   int inhibit;
   int inhibit_lvl;
 
+  int toggle_lvl; /* backlight level for simple toggle */
+
   int auto_on;  /* automatic */
   int r_sens;   /* right sensor */
   int l_sens;   /* left sensor */
@@ -62,6 +58,9 @@ kbd_backlight_fix_config(void);
 
 
 /* In kbd_auto.c */
+void
+kbd_backlight_toggle(void);
+
 void
 kbd_backlight_inhibit_set(int mask);
 
