@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: kbd_backlight.c 291 2007-03-02 19:45:47Z jblache $
+ * $Id: kbd_backlight.c 336 2007-06-17 08:44:45Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -26,6 +26,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
+#include <time.h>
 
 #include <syslog.h>
 
@@ -45,7 +46,8 @@ int
 has_kbd_backlight(void)
 {
   return ((mops->type == MACHINE_MACBOOKPRO_1)
-	  || (mops->type == MACHINE_MACBOOKPRO_2));
+	  || (mops->type == MACHINE_MACBOOKPRO_2)
+	  || (mops->type == MACHINE_MACBOOKPRO_3));
 }
 
 
@@ -87,9 +89,14 @@ kbd_backlight_set(int val, int who)
 {
   int curval;
 
+  int i;
+  float fadeval;
+  float step;
+  struct timespec fade_step;
+
   FILE *fp;
 
-  if (kbd_bck_info.inhibit)
+  if (kbd_bck_info.inhibit & ~KBD_INHIBIT_CFG)
     return;
 
   curval = kbd_backlight_get();
@@ -100,6 +107,35 @@ kbd_backlight_set(int val, int who)
   if ((val < KBD_BACKLIGHT_OFF) || (val > KBD_BACKLIGHT_MAX))
     return;
 
+  if (who == KBD_AUTO)
+    {
+      fade_step.tv_sec = 0;
+      fade_step.tv_nsec = (KBD_BACKLIGHT_FADE_LENGTH / KBD_BACKLIGHT_FADE_STEPS) * 1000000;
+
+      fadeval = (float)curval;
+      step = (float)(val - curval) / (float)KBD_BACKLIGHT_FADE_STEPS;
+
+      for (i = 0; i < KBD_BACKLIGHT_FADE_STEPS; i++)
+	{
+	  fadeval += step;
+
+	  fp = fopen(KBD_BACKLIGHT, "a");
+	  if (fp == NULL)
+	    {
+	      logmsg(LOG_WARNING, "Could not open %s: %s", KBD_BACKLIGHT, strerror(errno));
+	      continue;
+	    }
+
+	  fprintf(fp, "%d", (int)fadeval);
+
+	  fclose(fp);
+
+	  logdebug("KBD backlight value faded to %d\n", (int)fadeval);
+
+	  nanosleep(&fade_step, NULL);
+	}
+    }
+
   fp = fopen(KBD_BACKLIGHT, "a");
   if (fp == NULL)
     {
@@ -124,7 +160,7 @@ kbd_backlight_step(int dir)
   int val;
   int newval;
 
-  if (kbd_bck_info.inhibit)
+  if (kbd_bck_info.inhibit & ~KBD_INHIBIT_CFG)
     return;
 
   val = kbd_backlight_get();
@@ -167,7 +203,9 @@ kbd_backlight_init(void)
   if (kbd_cfg.auto_on)
     kbd_bck_info.inhibit = 0;
   else
-    kbd_bck_info.inhibit = KBD_INHIBIT_USER;
+    kbd_bck_info.inhibit = KBD_INHIBIT_CFG;
+
+  kbd_bck_info.toggle_lvl = kbd_cfg.auto_lvl;
 
   kbd_bck_info.inhibit_lvl = 0;
 
