@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: pommed.c 290 2007-03-02 19:34:39Z jblache $
+ * $Id: pommed.c 345 2007-07-27 14:00:46Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -53,6 +53,7 @@
 #include "conffile.h"
 #include "audio.h"
 #include "dbus.h"
+#include "power.h"
 
 
 /* Machine-specific operations */
@@ -74,6 +75,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_32,
     .lcd_backlight_probe = aty128_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step_kernel,
+    .lcd_backlight_toggle = sysfs_backlight_toggle_kernel,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -81,6 +83,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_33,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -88,6 +91,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_34,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -95,6 +99,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_35,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -104,6 +109,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_51,
     .lcd_backlight_probe = nvidia_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -111,6 +117,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_52,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -118,6 +125,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_53,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -125,6 +133,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_54,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -132,6 +141,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_55,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -139,6 +149,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_56,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_fountain,
   },
 
@@ -146,6 +157,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_57,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_fountain,
   },
 
@@ -153,14 +165,16 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_58,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
-    .evdev_identify = evdev_is_fountain,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
+    .evdev_identify = evdev_is_geyser,
   },
 
   {  /* PowerBook5,9 */
     .type = MACHINE_POWERBOOK_59,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
-    .evdev_identify = evdev_is_fountain,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
+    .evdev_identify = evdev_is_geyser,
   },
 
   /* G4 iBooks & 12" PowerBooks */
@@ -169,6 +183,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_61,
     .lcd_backlight_probe = nvidia_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -176,6 +191,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_62,
     .lcd_backlight_probe = nvidia_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -183,6 +199,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_63,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -190,6 +207,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_64,
     .lcd_backlight_probe = nvidia_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -197,6 +215,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_65,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -206,6 +225,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_67,
     .lcd_backlight_probe = r9x00_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   },
 
@@ -213,6 +233,7 @@ struct machine_ops pb_mops[] = {
     .type = MACHINE_POWERBOOK_68,
     .lcd_backlight_probe = nvidia_sysfs_backlight_probe,
     .lcd_backlight_step = sysfs_backlight_step,
+    .lcd_backlight_toggle = sysfs_backlight_toggle,
     .evdev_identify = evdev_is_adb,
   }
 };
@@ -226,6 +247,7 @@ struct machine_ops mb_mops[] = {
     .type = MACHINE_MACBOOKPRO_1,
     .lcd_backlight_probe = x1600_backlight_probe,
     .lcd_backlight_step = x1600_backlight_step,
+    .lcd_backlight_toggle = x1600_backlight_toggle,
     .evdev_identify = evdev_is_geyser3,
   },
 
@@ -233,9 +255,17 @@ struct machine_ops mb_mops[] = {
     .type = MACHINE_MACBOOKPRO_2,
     .lcd_backlight_probe = x1600_backlight_probe,
     .lcd_backlight_step = x1600_backlight_step,
+    .lcd_backlight_toggle = x1600_backlight_toggle,
     .evdev_identify = evdev_is_geyser4,
   },
 
+  {  /* MacBookPro3,1 (15" & 17", Core2 Duo, June 2007) */
+    .type = MACHINE_MACBOOKPRO_3,
+    .lcd_backlight_probe = nv8600mgt_backlight_probe,
+    .lcd_backlight_step = nv8600mgt_backlight_step,
+    .lcd_backlight_toggle = nv8600mgt_backlight_toggle,
+    .evdev_identify = evdev_is_geyser4,
+  },
 
   /* MacBook machines */
 
@@ -243,6 +273,7 @@ struct machine_ops mb_mops[] = {
     .type = MACHINE_MACBOOK_1,
     .lcd_backlight_probe = gma950_backlight_probe,
     .lcd_backlight_step = gma950_backlight_step,
+    .lcd_backlight_toggle = gma950_backlight_toggle,
     .evdev_identify = evdev_is_geyser3,
   },
 
@@ -250,6 +281,7 @@ struct machine_ops mb_mops[] = {
     .type = MACHINE_MACBOOK_2,
     .lcd_backlight_probe = gma950_backlight_probe,
     .lcd_backlight_step = gma950_backlight_step,
+    .lcd_backlight_toggle = gma950_backlight_toggle,
     .evdev_identify = evdev_is_geyser4,
   }
 };
@@ -500,7 +532,8 @@ check_machine_smbios(void)
 
   logdebug("SMBIOS vendor name: [%s]\n", prop);
 
-  if (strcmp(prop, "Apple Computer, Inc.") == 0)
+  if ((strcmp(prop, "Apple Computer, Inc.") == 0)
+      || (strcmp(prop, "Apple Inc.") == 0))
     ret = MACHINE_MAC_UNKNOWN;
 
   SMBIOSFreeMemory(prop);
@@ -525,6 +558,9 @@ check_machine_smbios(void)
   /* Core2 Duo MacBook Pro 17" & 15" (October 2006) */
   else if ((strcmp(prop, "MacBookPro2,1") == 0) || (strcmp(prop, "MacBookPro2,2") == 0))
     ret = MACHINE_MACBOOKPRO_2;
+  /* Core2 Duo MacBook Pro 15" & 17" (June 2007) */
+  else if (strcmp(prop, "MacBookPro3,1") == 0)
+    ret = MACHINE_MACBOOKPRO_3;
   /* Core Duo MacBook (May 2006) */
   else if (strcmp(prop, "MacBook1,1") == 0)
     ret = MACHINE_MACBOOK_1;
@@ -547,8 +583,8 @@ check_machine_smbios(void)
 static void
 usage(void)
 {
-  printf("pommed v" M_VERSION " ($Rev: 290 $) Apple laptops hotkeys handler\n");
-  printf("Copyright (C) 2006 Julien BLACHE <jb@jblache.org>\n");
+  printf("pommed v" M_VERSION " ($Rev: 345 $) Apple laptops hotkeys handler\n");
+  printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>\n");
 
   printf("Usage:\n");
   printf("\tpommed\t-- start pommed as a daemon\n");
@@ -597,7 +633,7 @@ main (int argc, char **argv)
 	    break;
 
 	  case 'v':
-	    printf("pommed v" M_VERSION " ($Rev: 290 $) Apple laptops hotkeys handler\n");
+	    printf("pommed v" M_VERSION " ($Rev: 345 $) Apple laptops hotkeys handler\n");
 	    printf("Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>\n");
 
 	    exit(0);
@@ -616,7 +652,7 @@ main (int argc, char **argv)
       openlog("pommed", LOG_PID, LOG_DAEMON);
     }
 
-  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 290 $) Apple laptops hotkeys handler");
+  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 345 $) Apple laptops hotkeys handler");
   logmsg(LOG_INFO, "Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>");
 
   /* Load our configuration */
@@ -697,6 +733,8 @@ main (int argc, char **argv)
       logmsg(LOG_WARNING, "Could not connect to DBus system bus");
     }
 
+  power_init();
+
   if (!console)
     {
       /*
@@ -782,36 +820,45 @@ main (int argc, char **argv)
 		evdev_process_events(fds[i].fd);
 	    }
 
-	  if (has_kbd_backlight())
+	  /* is it time to chek the ambient light sensors and AC state ? */
+	  gettimeofday(&tv_now, NULL);
+	  tv_diff.tv_sec = tv_now.tv_sec - tv_als.tv_sec;
+	  if (tv_diff.tv_sec < 0)
+	    tv_diff.tv_sec = 0;
+
+	  if (tv_diff.tv_sec == 0)
 	    {
-	      /* is it time to chek the ambient light sensors ? */
-	      gettimeofday(&tv_now, NULL);
-	      tv_diff.tv_sec = tv_now.tv_sec - tv_als.tv_sec;
-	      if (tv_diff.tv_sec < 0)
-		tv_diff.tv_sec = 0;
+	      tv_diff.tv_usec = tv_now.tv_usec - tv_als.tv_usec;
+	    }
+	  else
+	    {
+	      tv_diff.tv_sec--;
+	      tv_diff.tv_usec = 1000000 - tv_als.tv_usec + tv_now.tv_usec;
+	      tv_diff.tv_usec += tv_diff.tv_sec * 1000000;
+	    }
 
-	      if (tv_diff.tv_sec == 0)
-		{
-		  tv_diff.tv_usec = tv_now.tv_usec - tv_als.tv_usec;
-		}
-	      else
-		{
-		  tv_diff.tv_sec--;
-		  tv_diff.tv_usec = 1000000 - tv_als.tv_usec + tv_now.tv_usec;
-		  tv_diff.tv_usec += tv_diff.tv_sec * 1000000;
-		}
-
-	      if (tv_diff.tv_usec >= (1000 * LOOP_TIMEOUT))
+	  if (tv_diff.tv_usec >= (1000 * LOOP_TIMEOUT))
+	    {
+	      if (has_kbd_backlight())
 		{
 		  kbd_backlight_ambient_check();
-		  tv_als = tv_now;
 		}
+
+	      power_check_ac_state();
+
+	      tv_als = tv_now;
 	    }
 	}
-      else if (has_kbd_backlight())
+      else
 	{
-	  /* poll() timed out, check ambient light sensors */
-	  kbd_backlight_ambient_check();
+	  /* poll() timed out, check ambient light sensors and AC state */
+	  if (has_kbd_backlight())
+	    {
+	      kbd_backlight_ambient_check();
+	    }
+
+	  power_check_ac_state();
+
 	  gettimeofday(&tv_als, NULL);
 	}
 
