@@ -1,12 +1,12 @@
 /*
- * $Id: pommed.h 274 2007-02-23 11:54:43Z jblache $
+ * $Id: pommed.h 340 2007-07-26 15:23:27Z jblache $
  */
 
 #ifndef __POMMED_H__
 #define __POMMED_H__
 
 
-#define M_VERSION "1.2"
+#define M_VERSION "1.8"
 
 
 extern int debug;
@@ -32,6 +32,7 @@ typedef enum
 #ifndef __powerpc__
     MACHINE_MACBOOKPRO_1,
     MACHINE_MACBOOKPRO_2,
+    MACHINE_MACBOOKPRO_3,
     MACHINE_MACBOOK_1,
     MACHINE_MACBOOK_2,
 #else
@@ -66,6 +67,7 @@ struct machine_ops
   machine_type type;
   int (*lcd_backlight_probe) (void);
   void (*lcd_backlight_step) (int dir);
+  void (*lcd_backlight_toggle) (int lvl);
   int (*evdev_identify) (unsigned short *id);
 };
 
